
//------------------ Heading and Compass --------------------------------

static char buf_show[12];
const char buf_Rule[36] = {0x82,0x80,0x81,0x80,0x81,0x80,
                           0x84,0x80,0x81,0x80,0x81,0x80,
                           0x83,0x80,0x81,0x80,0x81,0x80,
                           0x85,0x80,0x81,0x80,0x81,0x80};

void setHeadingPattern()
{
  int start;
  start = round((osd_heading * 24)/360);
  start -= 3;
  if(start < 0) start += 24;
  for(int x=0; x <= 10; x++){
    buf_show[x] = buf_Rule[start];
    if(++start > 23) start = 0;
  }
  buf_show[7] = '\0';
}

//------------------ Battery Remaining Picture --------------------------

char setBatteryPic(uint16_t bat_level)
{
  if(bat_level <= 100){
    return 0xb4;
  }
  else if(bat_level <= 300){
    return 0xb5;
  }
  else if(bat_level <= 400){
    return 0xb6;
  }
  else if(bat_level <= 500){
    return 0xb7;
  }
  else if(bat_level <= 800){
    return 0xb8;
  }
  else return 0xb9;
}

//-------------- Home Distance and Direction Calculation ----------------

void setHomeVars(OSD &osd)
{
  float dstlon, dstlat;
  long bearing;
  
  // osd_alt_to_home = (osd_alt - osd_home_alt);

  //Check disarm to arm switching.
  if (motor_armed && !last_armed_status)
  {
    // If motors get armed, reset home in Arducopter version
    // Force a new home position to be read after motor arming
    osd_got_home = 0;
    has_armed    = 1;
  }

  last_armed_status = motor_armed;

  // Set a default; Arrow will not be shown unless we have
  // at least 2D positioning from GPS
  osd_home_direction = 0;

  // GPS lock 0-1=no fix, 2=2D, 3=3D
  if(osd_got_home == 0 && osd_fix_type == 3) {
    osd_home_lat = osd_lat;
    osd_home_lon = osd_lon;
    //osd_home_alt = osd_alt;
    osd_got_home = 1;
  } else {
    if(osd_got_home == 1 && osd_fix_type > 1) {
        float  rads = fabs(osd_home_lat) * 0.0174532925;
        double scaleLongDown = cos(rads);
        double scaleLongUp   = 1.0f/cos(rads);

        //DST to Home
        dstlat = fabs(osd_home_lat - osd_lat) * 111319.5;
        dstlon = fabs(osd_home_lon - osd_lon) * 111319.5 * scaleLongDown;
        osd_home_distance = sqrt(sq(dstlat) + sq(dstlon));

        //DIR to Home
        // 16 sectors, numbered 1 to 16, each is 22.5 degrees wide
        dstlon = (osd_home_lon - osd_lon);      //OffSet_X
        dstlat = (osd_home_lat - osd_lat) * scaleLongUp; //OffSet Y
        bearing = 90 + (atan2(dstlat, -dstlon) * 57.295775); //absolute home direction
        if(bearing < 0) bearing += 360;		//normalization
        bearing = bearing - 180;		//absolute return direction
        if(bearing < 0) bearing += 360;		//normalization
        //bearing = bearing - osd_heading;	//relative home direction
        bearing = bearing - osd_heading - 11;	// Arrow points to middle of sector
        while(bearing < 0) bearing += 360;	//normalization
        //osd_home_direction = round((float)(bearing/360.0f) * 16.0f) + 1; //array of arrows
        osd_home_direction = round((float)(bearing * 0.04444f)) + 1; //array of arrows
        if(osd_home_direction > 16) osd_home_direction -= 16;
    }
  }
}

void setFdataVars()
{
  // Moved from panel because warnings also need this var
  // and panClimb could be off
  vs = (osd_climb * converth * 0.1) + (vs * 0.9); // meters per second

// Make a moving average for battery voltage
  osd_vbat_A = (float)((osd_mvolt + ova + ovb + ovc) / 4000.0f);
  ova = ovb; ovb = ovc; ovc = osd_mvolt;

  if(max_battery_reading < osd_battery_remaining_A)
    max_battery_reading = osd_battery_remaining_A;

  unsigned long time_lapse = millis() - runtime;
  runtime = millis();

  // Add to odometer distance only when motors are armed
  if (motor_armed && osd_groundspeed > 1.0) tdistance += (osd_groundspeed * (time_lapse) / 1000.0);
  mah_used += (osd_curr_A * 10.0 * (time_lapse) / 3600000.0);

  // Set max data
  // osd_airspeed and osd_windspeed disabled for copter
  if (motor_armed)
  {
    total_flight_time_milis  += time_lapse;
    total_flight_time_seconds = total_flight_time_milis / 1000;
    if (osd_home_distance > max_home_distance) max_home_distance = osd_home_distance;
    //if (osd_airspeed > max_osd_airspeed) max_osd_airspeed = osd_airspeed;
    if (osd_groundspeed > max_osd_groundspeed) max_osd_groundspeed = osd_groundspeed;
    if (osd_alt_rel > max_osd_home_alt) max_osd_home_alt = osd_alt_rel;
    //if (osd_windspeed > max_osd_windspeed) max_osd_windspeed = osd_windspeed;
  } else {
    total_ground_time_milis  += time_lapse;
    total_ground_time_seconds = total_ground_time_milis / 1000;
  }
}

void checkModellType(){
if (EEPROM.read(MODELL_TYPE_ADD)  != 1) EEPROM.write(MODELL_TYPE_ADD,  1);
if (EEPROM.read(FW_VERSION1_ADDR) != 2) EEPROM.write(FW_VERSION1_ADDR, 2);
if (EEPROM.read(FW_VERSION2_ADDR) != 4) EEPROM.write(FW_VERSION2_ADDR, 4);
if (EEPROM.read(FW_VERSION3_ADDR) != 1) EEPROM.write(FW_VERSION3_ADDR, 1);
}

